@@ -10,11 +10,16 @@ import time
 import sys
 import io
 import os
+from math import sqrt
+
+from joblib import parallel
 
 from joblib.test.common import np, with_numpy
 from joblib.test.common import with_multiprocessing
 from joblib.testing import check_subprocess_call
 from joblib._compat import PY3_OR_LATER
+from multiprocessing import TimeoutError
+from time import sleep
 
 try:
     import cPickle as pickle
@@ -40,15 +45,23 @@ except ImportError:
     from Queue import Queue
 
 
-from joblib.parallel import Parallel, delayed, SafeFunction, WorkerInterrupt
-from joblib.parallel import mp, cpu_count, VALID_BACKENDS
+from joblib._parallel_backends import SequentialBackend
+from joblib._parallel_backends import ThreadingBackend
+from joblib._parallel_backends import MultiprocessingBackend
+from joblib._parallel_backends import SafeFunction
+from joblib._parallel_backends import WorkerInterrupt
+
+from joblib.parallel import Parallel, delayed
+from joblib.parallel import register_parallel_backend, parallel_backend
+
+from joblib.parallel import mp, cpu_count, BACKENDS, effective_n_jobs
 from joblib.my_exceptions import JoblibException
 
 import nose
 from nose.tools import assert_equal, assert_true, assert_false, assert_raises
 
 
-ALL_VALID_BACKENDS = [None] + VALID_BACKENDS
+ALL_VALID_BACKENDS = [None] + sorted(BACKENDS.keys())
 
 if hasattr(mp, 'get_context'):
     # Custom multiprocessing context in Python 3.4+
@@ -89,11 +102,19 @@ def f(x, y=0, z=0):
     return x ** 2 + y + z
 
 
+def _active_backend_type():
+    return type(parallel.get_active_backend()[0])
+
+
 ###############################################################################
 def test_cpu_count():
     assert cpu_count() > 0
 
 
+def test_effective_n_jobs():
+    assert effective_n_jobs() > 0
+
+
 ###############################################################################
 # Test parallel
 def check_simple_parallel(backend):
@@ -101,7 +122,8 @@ def check_simple_parallel(backend):
     for n_jobs in (1, 2, -1, -2):
         assert_equal(
             [square(x) for x in X],
-            Parallel(n_jobs=n_jobs)(delayed(square)(x) for x in X))
+            Parallel(n_jobs=n_jobs, backend=backend)(
+                delayed(square)(x) for x in X))
     try:
         # To smoke-test verbosity, we capture stdout
         orig_stdout = sys.stdout
@@ -151,8 +173,8 @@ def check_nested_loop(parent_backend, child_backend):
 
 
 def test_nested_loop():
-    for parent_backend in VALID_BACKENDS:
-        for child_backend in VALID_BACKENDS:
+    for parent_backend in BACKENDS:
+        for child_backend in BACKENDS:
             yield check_nested_loop, parent_backend, child_backend
 
 
@@ -173,15 +195,16 @@ def test_parallel_kwargs():
                Parallel(n_jobs=n_jobs)(delayed(f)(x, y=1) for x in lst))
 
 
-def check_parallel_context_manager(backend):
+def check_parallel_as_context_manager(backend):
     lst = range(10)
     expected = [f(x, y=1) for x in lst]
     with Parallel(n_jobs=4, backend=backend) as p:
         # Internally a pool instance has been eagerly created and is managed
         # via the context manager protocol
-        managed_pool = p._pool
+        managed_backend = p._backend
         if mp is not None:
-            assert_true(managed_pool is not None)
+            assert_true(managed_backend is not None)
+            assert_true(managed_backend._pool is not None)
 
         # We make call with the managed parallel object several times inside
         # the managed block:
@@ -190,20 +213,22 @@ def check_parallel_context_manager(backend):
 
         # Those calls have all used the same pool instance:
         if mp is not None:
-            assert_true(managed_pool is p._pool)
+            assert_true(managed_backend._pool is p._backend._pool)
 
     # As soon as we exit the context manager block, the pool is terminated and
     # no longer referenced from the parallel object:
-    assert_true(p._pool is None)
+    if mp is not None:
+        assert_true(p._backend._pool is None)
 
     # It's still possible to use the parallel instance in non-managed mode:
     assert_equal(expected, p(delayed(f)(x, y=1) for x in lst))
-    assert_true(p._pool is None)
+    if mp is not None:
+        assert_true(p._backend._pool is None)
 
 
 def test_parallel_context_manager():
     for backend in ['multiprocessing', 'threading']:
-        yield check_parallel_context_manager, backend
+        yield check_parallel_as_context_manager, backend
 
 
 def test_parallel_pickling():
@@ -225,6 +250,25 @@ def test_parallel_pickling():
                   (delayed(g)(x) for x in range(10)))
 
 
+def test_parallel_timeout_success():
+    # Check that timeout isn't thrown when function is fast enough
+    for backend in ['multiprocessing', 'threading']:
+        nose.tools.assert_equal(
+            10,
+            len(Parallel(n_jobs=2, backend=backend, timeout=10)
+                (delayed(sleep)(0.001) for x in range(10))))
+
+
+def test_parallel_timeout_fail():
+    # Check that timeout properly fails when function is too slow
+    for backend in ['multiprocessing', 'threading']:
+        nose.tools.assert_raises(
+            TimeoutError,
+            Parallel(n_jobs=2, backend=backend, timeout=0.01),
+            (delayed(sleep)(10) for x in range(10))
+        )
+
+
 def test_error_capture():
     # Check that error are captured, and that correct exceptions
     # are raised.
@@ -239,7 +283,8 @@ def test_error_capture():
 
         # Try again with the context manager API
         with Parallel(n_jobs=2) as parallel:
-            assert_true(parallel._pool is not None)
+            assert_true(parallel._backend._pool is not None)
+            original_pool = parallel._backend._pool
 
             assert_raises(JoblibException, parallel,
                           [delayed(division)(x, y)
@@ -247,21 +292,30 @@ def test_error_capture():
 
             # The managed pool should still be available and be in a working
             # state despite the previously raised (and caught) exception
-            assert_true(parallel._pool is not None)
+            assert_true(parallel._backend._pool is not None)
+
+            # The pool should have been interrupted and restarted:
+            assert_true(parallel._backend._pool is not original_pool)
+
             assert_equal([f(x, y=1) for x in range(10)],
                          parallel(delayed(f)(x, y=1) for x in range(10)))
 
+            original_pool = parallel._backend._pool
             assert_raises(WorkerInterrupt, parallel,
                           [delayed(interrupt_raiser)(x) for x in (1, 0)])
 
             # The pool should still be available despite the exception
-            assert_true(parallel._pool is not None)
+            assert_true(parallel._backend._pool is not None)
+
+            # The pool should have been interrupted and restarted:
+            assert_true(parallel._backend._pool is not original_pool)
+
             assert_equal([f(x, y=1) for x in range(10)],
                          parallel(delayed(f)(x, y=1) for x in range(10)))
 
         # Check that the inner pool has been terminated when exiting the
         # context manager
-        assert_true(parallel._pool is None)
+        assert_true(parallel._backend._pool is None)
     else:
         assert_raises(KeyboardInterrupt, Parallel(n_jobs=2),
                       [delayed(interrupt_raiser)(x) for x in (1, 0)])
@@ -341,7 +395,7 @@ def check_dispatch_one_job(backend):
 
 
 def test_dispatch_one_job():
-    for backend in VALID_BACKENDS:
+    for backend in BACKENDS:
         yield check_dispatch_one_job, backend
 
 
@@ -374,40 +428,37 @@ def check_dispatch_multiprocessing(backend):
 
 
 def test_dispatch_multiprocessing():
-    for backend in VALID_BACKENDS:
-        yield check_dispatch_multiprocessing, backend
+    for backend in BACKENDS:
+        if backend != "sequential":
+            yield check_dispatch_multiprocessing, backend
 
 
 def test_batching_auto_threading():
     # batching='auto' with the threading backend leaves the effective batch
     # size to 1 (no batching) as it has been found to never be beneficial with
     # this low-overhead backend.
-    p = Parallel(n_jobs=2, batch_size='auto', backend='threading')
-    p(delayed(id)(i) for i in range(5000))  # many very fast tasks
-    assert_equal(p._effective_batch_size, 1)
+
+    with Parallel(n_jobs=2, batch_size='auto', backend='threading') as p:
+        p(delayed(id)(i) for i in range(5000))  # many very fast tasks
+        assert_equal(p._backend.compute_batch_size(), 1)
 
 
 def test_batching_auto_multiprocessing():
-    p = Parallel(n_jobs=2, batch_size='auto', backend='multiprocessing')
-    p(delayed(id)(i) for i in range(5000))  # many very fast tasks
+    with Parallel(n_jobs=2, batch_size='auto', backend='multiprocessing') as p:
+        p(delayed(id)(i) for i in range(5000))  # many very fast tasks
 
-    # When the auto-tuning of the batch size is enabled
-    # size kicks in the following attribute gets updated.
-    assert_true(hasattr(p, '_effective_batch_size'))
-
-    # It should be strictly larger than 1 but as we don't want heisen failures
-    # on clogged CI worker environment be safe and only check that it's a
-    # strictly positive number.
-    assert_true(p._effective_batch_size > 0)
+        # It should be strictly larger than 1 but as we don't want heisen
+        # failures on clogged CI worker environment be safe and only check that
+        # it's a strictly positive number.
+        assert_true(p._backend.compute_batch_size() > 0)
 
 
 def test_exception_dispatch():
     "Make sure that exception raised during dispatch are indeed captured"
     assert_raises(
-            ValueError,
-            Parallel(n_jobs=2, pre_dispatch=16, verbose=0),
-                    (delayed(exception_raiser)(i) for i in range(30)),
-            )
+        ValueError,
+        Parallel(n_jobs=2, pre_dispatch=16, verbose=0),
+        (delayed(exception_raiser)(i) for i in range(30)))
 
 
 def test_nested_exception_dispatch():
@@ -440,6 +491,142 @@ def test_multiple_spawning():
                   [delayed(_reload_joblib)() for i in range(10)])
 
 
+class FakeParallelBackend(SequentialBackend):
+    """Pretends to run concurrently while running sequentially."""
+
+    def configure(self, n_jobs=1, parallel=None, **backend_args):
+        self.n_jobs = self.effective_n_jobs(n_jobs)
+        self.parallel = parallel
+        return n_jobs
+
+    def effective_n_jobs(self, n_jobs=1):
+        if n_jobs < 0:
+            n_jobs = max(mp.cpu_count() + 1 + n_jobs, 1)
+        return n_jobs
+
+
+def test_invalid_backend():
+    assert_raises(ValueError, Parallel, backend='unit-testing')
+
+
+def test_register_parallel_backend():
+    try:
+        register_parallel_backend("test_backend", FakeParallelBackend)
+        assert_true("test_backend" in BACKENDS)
+        assert_equal(BACKENDS["test_backend"], FakeParallelBackend)
+    finally:
+        del BACKENDS["test_backend"]
+
+
+def test_overwrite_default_backend():
+    assert_equal(_active_backend_type(), MultiprocessingBackend)
+    try:
+        register_parallel_backend("threading", BACKENDS["threading"],
+                                  make_default=True)
+        assert_equal(_active_backend_type(), ThreadingBackend)
+    finally:
+        # Restore the global default manually
+        parallel.DEFAULT_BACKEND = 'multiprocessing'
+    assert_equal(_active_backend_type(), MultiprocessingBackend)
+
+
+def check_backend_context_manager(backend_name):
+    with parallel_backend(backend_name, n_jobs=3):
+        active_backend, active_n_jobs = parallel.get_active_backend()
+        assert_equal(active_n_jobs, 3)
+        assert_equal(effective_n_jobs(3), 3)
+        p = Parallel()
+        assert_equal(p.n_jobs, 3)
+        if backend_name == 'multiprocessing':
+            assert_equal(type(active_backend), MultiprocessingBackend)
+            assert_equal(type(p._backend), MultiprocessingBackend)
+        elif backend_name == 'threading':
+            assert_equal(type(active_backend), ThreadingBackend)
+            assert_equal(type(p._backend), ThreadingBackend)
+        elif backend_name.startswith('test_'):
+            assert_equal(type(active_backend), FakeParallelBackend)
+            assert_equal(type(p._backend), FakeParallelBackend)
+
+
+def test_backend_context_manager():
+    all_test_backends = ['test_backend_%d' % i for i in range(3)]
+    for test_backend in all_test_backends:
+        register_parallel_backend(test_backend, FakeParallelBackend)
+    all_backends = ['multiprocessing', 'threading'] + all_test_backends
+
+    try:
+        assert_equal(_active_backend_type(), MultiprocessingBackend)
+        # check that this possible to switch parallel backends sequentially
+        for test_backend in all_backends:
+            yield check_backend_context_manager, test_backend
+
+        # The default backend is retored
+        assert_equal(_active_backend_type(), MultiprocessingBackend)
+
+        # Check that context manager switching is thread safe:
+        Parallel(n_jobs=2, backend='threading')(
+            delayed(check_backend_context_manager)(b)
+            for b in all_backends if not b)
+
+        # The default backend is again retored
+        assert_equal(_active_backend_type(), MultiprocessingBackend)
+    finally:
+        for backend_name in list(BACKENDS.keys()):
+            if backend_name.startswith('test_'):
+                del BACKENDS[backend_name]
+
+
+class ParameterizedParallelBackend(SequentialBackend):
+    """Pretends to run conncurrently while running sequentially."""
+
+    def __init__(self, param=None):
+        if param is None:
+            raise ValueError('param should not be None')
+        self.param = param
+
+
+def test_parameterized_backend_context_manager():
+    register_parallel_backend('param_backend', ParameterizedParallelBackend)
+    try:
+        assert_equal(_active_backend_type(), MultiprocessingBackend)
+
+        with parallel_backend('param_backend', param=42, n_jobs=3):
+            active_backend, active_n_jobs = parallel.get_active_backend()
+            assert_equal(type(active_backend), ParameterizedParallelBackend)
+            assert_equal(active_backend.param, 42)
+            assert_equal(active_n_jobs, 3)
+            p = Parallel()
+            assert_equal(p.n_jobs, 3)
+            assert_true(p._backend is active_backend)
+            results = p(delayed(sqrt)(i) for i in range(5))
+        assert_equal(results, [sqrt(i) for i in range(5)])
+
+        # The default backend is again retored
+        assert_equal(_active_backend_type(), MultiprocessingBackend)
+    finally:
+        del BACKENDS['param_backend']
+
+
+def test_direct_parameterized_backend_context_manager():
+    assert_equal(_active_backend_type(), MultiprocessingBackend)
+
+    # Check that it's possible to pass a backend instance directly,
+    # without registration
+    with parallel_backend(ParameterizedParallelBackend(param=43), n_jobs=5):
+        active_backend, active_n_jobs = parallel.get_active_backend()
+        assert_equal(type(active_backend), ParameterizedParallelBackend)
+        assert_equal(active_backend.param, 43)
+        assert_equal(active_n_jobs, 5)
+        p = Parallel()
+        assert_equal(p.n_jobs, 5)
+        assert_true(p._backend is active_backend)
+        results = p(delayed(sqrt)(i) for i in range(5))
+    assert_equal(results, [sqrt(i) for i in range(5)])
+
+    # The default backend is again retored
+    assert_equal(_active_backend_type(), MultiprocessingBackend)
+
+
 ###############################################################################
 # Test helpers
 def test_joblib_exception():
@@ -490,20 +677,22 @@ def test_dispatch_race_condition():
 
 def test_default_mp_context():
     p = Parallel(n_jobs=2, backend='multiprocessing')
+    context = p._backend_args.get('context')
     if sys.version_info >= (3, 4):
+        start_method = context.get_start_method()
         # Under Python 3.4+ the multiprocessing context can be configured
         # by an environment variable
         env_method = os.environ.get('JOBLIB_START_METHOD', '').strip() or None
         if env_method is None:
             # Check the default behavior
             if sys.platform == 'win32':
-                assert_equal(p._mp_context.get_start_method(), 'spawn')
+                assert_equal(start_method, 'spawn')
             else:
-                assert_equal(p._mp_context.get_start_method(), 'fork')
+                assert_equal(start_method, 'fork')
         else:
-            assert_equal(p._mp_context.get_start_method(), env_method)
+            assert_equal(start_method, env_method)
     else:
-        assert_equal(p._mp_context, None)
+        assert_equal(context, None)
 
 
 @with_numpy
